/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaManagementPage");

dojo.require("dijit._Widget");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.ui.SideNavigationLayout");

dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ManageProjectAreas");
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ManageArchivedProjectAreas");
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaEditor");
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.TeamAreaEditor");

var ActionRegistry;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var SideNavigationLayout = net.jazz.ajax.ui.SideNavigationLayout;
var ManageProjectAreas = com.ibm.team.process.web.ui.internal.admin.projects.ManageProjectAreas;
var ManageArchivedProjectAreas = com.ibm.team.process.web.ui.internal.admin.projects.ManageArchivedProjectAreas;
var ProjectAreaEditor = com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaEditor;
var TeamAreaEditor = com.ibm.team.process.web.ui.internal.admin.projects.TeamAreaEditor;

var PREFIX = "com.ibm.team.process.";

var ACTION_SETS = [
	{ label: "Project Area Management", actions: [
		{label: "Active Project Areas", shortId: "manageProjectAreas", aliases: ["editProjectArea", "editTeamArea"], widget: "ManageProjectAreas", showInNav: true},
		{label: "Archived Project Areas", shortId: "manageArchivedProjectAreas", aliases: ["editArchivedProjectArea"], widget: "ManageArchivedProjectAreas", showInNav: true},
		{label: "Edit Project Area", shortId: "editProjectArea", widget: "ProjectAreaEditor", showInNav: false},
		{label: "Edit Archived Project Area", shortId: "editArchivedProjectArea", widget: "ProjectAreaEditor", showInNav: false},
		{label: "Edit Team Area", shortId: "editTeamArea", widget: "TeamAreaEditor", showInNav: false}
	]}
];
var ACTION_MAP = {};

function getNavigationLink (action, title) {
	var aElement = document.createElement("a");
	aElement.href = "#action=" + action;
	aElement.appendChild(document.createTextNode(title));
	return aElement;
}

function getWorkbench () {
	return PlatformUI.getWorkbench();
}

dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaManagementPage", dijit._Widget, {
	
	reloadCurrent: function() {
		this._previousActionEntry = this._currentActionEntry || null;
		this._showCurrentWidget(/*isReload*/true);
	},

	postCreate: function () {
		this.inherited(arguments);
		
		this._widgetCache = {};
		this.userEditorCache = {};
		
		this.domNode.className = "com-ibm-team-repository-admin";
		
		this._appView = new net.jazz.ajax.ui.SideNavigationLayout();
		this.domNode.appendChild(this._appView.domNode);
		
		// register actions and create hyperlinks
		ActionRegistry = getWorkbench().getActionRegistry();
		ActionRegistry.registerAction(PREFIX + "_defaultAction", this, "_defaultAction");
		
		var self = this;
		this.allLinks = [];
		dojo.forEach(ACTION_SETS, function(actionSet) {
			var navSection = document.createElement("div");
			self.connect(navSection, "onclick", self._onNavSectionClick);
			navSection.className = "com-ibm-team-repository-admin-navSection";
			self._appView.addNavigationSection(navSection, actionSet.label, true);
			dojo.forEach(actionSet.actions, function(action){
				var id = PREFIX + action.shortId;
				ACTION_MAP[action.shortId] = action;
				var link = getNavigationLink(id, action.label);
				link.__jazzActionId = action.shortId;
				link.aliases = action.aliases;
				if (action.showInNav) {
					navSection.appendChild(link);
				}
				var span = document.createElement("span");
				span.__jazzReload = true;
				link.appendChild(span);
				self.allLinks.push(link);
				ActionRegistry.registerAction(id, self, "_handleAction");
			});
		});
	},
	
	_defaultAction: function() {
		// by default, show the first section from the left-nav
		this._handleAction({action: PREFIX + ACTION_SETS[0].actions[0].shortId});
	},
	
	_initAdminUI: function () {
		// by default, show the first section from the left-nav
		this._handleAction({action: PREFIX + ACTIONS[0].shortId});
	},
	
	_handleAction: function(args) {
		if (!isAdmin()) {
			this._showUnauthorized();
			return;
		}

		if(!args.action) {
			console.debug("UsersPage.js#_handleAction: missing required param 'args.action'");
			return;
		}
		
		// cache the previous entry to see if the widget should be destroyed or
		// not
		this._previousActionEntry = this._currentActionEntry || null;
		
		// parse out the short id from the fully-qualified id
		var shortId = args.action.substring(args.action.lastIndexOf(".") + 1);
		this._currentActionEntry = ACTION_MAP[shortId];
		this._currentArgs = args;
		this._currentArgs.parent = this;
		
		// set the title for the action
		getWorkbench().setTitle(this._currentActionEntry.label);
		
		// highlight the selected link and unhighlight the others
		dojo.forEach(this.allLinks, function(link){
			if (link.__jazzActionId === shortId) {
				dojo.toggleClass(link, "selected", true);
			}
			else {
				if (link.aliases != null) {
					var found = false;
					for (var i=0; i < link.aliases.length; i++) {
						if (link.aliases[i] === shortId) {
							dojo.toggleClass(link, "selected", true);
							found = true;
							break;
						}
					}
					if (!found) {
						dojo.toggleClass(link, "selected", false);
					}
				}
				else {
					dojo.toggleClass(link, "selected", false);
				}
			}
		});
		
		// call the actual method implementation
		if (!this._currentActionEntry.simpleAction || this._currentContentWidget == null ||
				this._currentContentWidget.id.replace(/_\d+$/,"").replace(/_/g,".") !== SERVER_PACKAGE + this._currentActionEntry.widget) {
			this._showCurrentWidget();
		}
		if (this._currentActionEntry.simpleAction) {
			this._currentActionEntry.simpleAction(args, this._currentContentWidget);
		}
	},
	
	_onNavSectionClick: function(e) {
		if(e.target.__jazzReload) {
			dojo.stopEvent(e);
			this.reloadCurrent();
		}
	},
	
	_showCurrentWidget: function(isReload) {
		// save a reference to the prior content widget
		this._previousContentWidget = this._currentContentWidget || null;
		
		if(this._currentActionEntry.cached) {
			this._showCachedWidget(isReload);
		} else {
			this._showUncachedWidget();
		}
		
		// help the garbage collector do its job
		if(this._previousContentWidget && !this._previousActionEntry.cached) {
			this._previousContentWidget.destroyRecursive();
		}
	},
	
	_showCachedWidget: function(isReload) {
		var className = this._currentActionEntry.widget;
		if(this._widgetCache[className]) {
			// get widget from the cache if it exists
			this._currentContentWidget = this._widgetCache[className];
		} else {
			// otherwise instantiate it and add it to the cache
			var constructor = dojo.getObject(SERVER_PACKAGE + className);
			this._currentContentWidget = new constructor(this._currentArgs);
			dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea");
			this._widgetCache[className] = this._currentContentWidget;
		}
		if(isReload) {
			this._currentContentWidget.reload(this._currentArgs);
		} else {
			this._currentContentWidget.show(this._currentArgs);
			this._appView.setMainContent(this._currentContentWidget.domNode);
		}
	},
	
	_showUncachedWidget: function() {
		var constructor = dojo.getObject(SERVER_PACKAGE + this._currentActionEntry.widget);
		this._currentContentWidget = new constructor(this._currentArgs);
		dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea");
		this._appView.setMainContent(this._currentContentWidget.domNode);
	},
	
	_showUnauthorized: function() {
		if (!this._unauth) {
			this._unauth = new NoAuth();
		}
		dojox.data.dom.removeChildren(this.domNode);
		this.domNode.appendChild(this._unauth.domNode);
	}
	
});

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.NoAuth", [dijit._Widget, dijit._Templated], {

	templateString: "<div class=\"com-ibm-team-repository-admin-noAuth\"><p>You do not have the required role to access the Jazz Team Server Admin.</p></div>",

	postCreate: function() {
		this.inherited(arguments);
	}
});

var NoAuth = com.ibm.team.repository.web.ui.internal.admin.NoAuth;

var SERVER_PACKAGE = "com.ibm.team.process.web.ui.internal.admin.projects.";

function isAdmin() {
	return com.ibm.team.repository.web.noAdminAuth ? false : true;
}

var ProjectAreaManagementPage = com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaManagementPage;

})();
